<?php
namespace Ak\Lib\File;

use finfo;

/**
 * Osnovna datoteka.
 *
 * @package Ak\Lib\File
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/lib-file
 */
class File implements FileInterface
{
    /** @var string */
    private $contents;
    /** @var string */
    private $md5;
    /** @var string */
    private $mimeType;
    /** @var string */
    private $name;
    /** @var resource */
    private $stream;

    /**
     * Magični brojevi za detekciju mime-typea.
     *
     * @var array
     */
    private static $mimeMagicNumbers = [
        '\x47\x49\x46\x38\x37\x61'                         => 'image/gif',
        '\x47\x49\x46\x38\x39\x61'                         => 'image/gif',
        '\xFF\xD8\xFF\xDB'                                 => 'image/jpeg',
        '\xFF\xD8\xFF\xE0\x00\x10\x4A\x46\x49\x46\x00\x01' => 'image/jpeg',
        '\xFF\xD8\xFF\xEE'                                 => 'image/jpeg',
        '\xFF\xD8\xFF\xE1..\x45\x78\x69\x66\x00\x00'       => 'image/jpeg',
        '\x89\x50\x4E\x47\x0D\x0A\x1A\x0A'                 => 'image/png',
    ];

    /**
     * Konstruktor.
     *
     * @param string|resource $contents
     * @param string|null $name
     */
    public function __construct($contents = null, string $name = null)
    {
        if (is_resource($contents)) {
            $this->stream = $contents;
        } else {
            $this->contents = $contents ?? '';
        }

        $this->name = $name;
    }

    /**
     * Destruktor.
     */
    public function __destruct()
    {
        if (is_resource($this->stream)) {
            fclose($this->stream);
        }
    }

    /**
     * Vraća sadržaj datoteke.
     *
     * @return string
     */
    public function getContents(): string
    {
        if (null === $this->contents) {
            $stream = $this->getStream();
            $this->contents = stream_get_contents($stream);
        }

        return $this->contents;
    }


    /**
     * Vraća MD5 hash ove datoteke.
     *
     * @return string
     */
    public function getMd5Hash(): string
    {
        if (null === $this->md5) {
            $this->md5 = md5($this->getContents());
        }

        return $this->md5;
    }

    /**
     * Vraća mime-type ove datoteke.
     *
     * @return string
     */
    public function getMimeType(): string
    {
        if (null === $this->mimeType) {
            if (null === $this->contents) {
                $header = fread($this->stream, 12);
                rewind($this->stream);
            } else {
                $header = substr($this->contents, 0, 12);
            }

            foreach (self::$mimeMagicNumbers as $magicNumber => $mimeType) {
                if (preg_match('/^' . $magicNumber . '/', $header)) {
                    $this->mimeType = $mimeType;
                }
            }
        }

        if (null === $this->mimeType) {
            $finfo = new finfo(FILEINFO_MIME_TYPE);
            $this->mimeType = preg_replace('/;.+$/', '', $finfo->buffer($this->getContents()));
        }

        return $this->mimeType;
    }

    /**
     * Vraća ime ove datoteke.
     *
     * @return string|null
     */
    public function getName(): ?string
    {
        return $this->name;
    }

    /**
     * Postavlja ime ove datoteke.
     *
     * @param string $name
     */
    public function setName(string $name): void
    {
        $this->name = $name;
    }

    /**
     * Vraća stream za ovu datoteku.
     *
     * @return resource
     */
    public function getStream()
    {
        if (null === $this->stream) {
            $this->stream = fopen('php://memory', 'w');

            if (null !== $this->contents) {
                fputs($this->stream, $this->contents);
            }
        }

        rewind($this->stream);
        return $this->stream;
    }
}